/**
 * DO NOT EDIT
 *
 * This file was automatically generated by
 *   https://github.com/Polymer/gen-typescript-declarations
 *
 * To modify these typings, edit the source file(s):
 *   lib/utils/telemetry.html
 */

/// <reference path="boot.d.ts" />

declare namespace Polymer {

  /**
   * Provides basic tracking of element definitions (registrations) and
   * instance counts.
   */
  namespace telemetry {
  }
}
