/**
 * DO NOT EDIT
 *
 * This file was automatically generated by
 *   https://github.com/Polymer/gen-typescript-declarations
 *
 * To modify these typings, edit the source file(s):
 *   lib/legacy/polymer-fn.html
 */

/// <reference path="class.d.ts" />


/**
 * Legacy class factory and registration helper for defining Polymer
 * elements.
 *
 * This method is equivalent to
 * `customElements.define(info.is, Polymer.Class(info));`
 *
 * See `Polymer.Class` for details on valid legacy metadata format for `info`.
 *
 * @returns Generated class
 */
declare function Polymer(info: PolymerInit): {new(): HTMLElement};
